@@ -2,7 +2,7 @@
 /*
  *
  * $RCSfile: blackboard.php,v $
- * $Date: 2005/07/04 12:42:46 $
+ * $Date: 2005/07/28 15:04:18 $
  * $Author: die $
  * 
  * Ф󥵡¦ץ
@@ -32,8 +32,12 @@
 		plugin_activate($_REQUEST['plugin']);
 
 	} else if ($_SERVER['REQUEST_METHOD'] == 'GET') {
-		plugins_initialize();
-		require_once('blackboard.html.inc');
+		if (!file_exists($conf['conf_dir'] . 'adminpass')) {
+			require_once('postinstall.inc');
+		} else {
+			plugins_initialize();
+			require_once('blackboard.html.inc');
+		}
 
 	// 
 	} else if (isset($_POST['erase']) && $_POST['erase'] == 'erase') {
@@ -45,6 +49,11 @@
 		$t = image_update($_POST['s']);
 		header('content-type: text/plain');
 		echo $t;
+
+	// ԥѥɤ
+	} else if (isset($_POST['admin_password'])) {
+		require_once('postinstall.inc');
+
 	// ¾
 	} else {
 		header('content-type: text/plain');
@@ -56,8 +65,8 @@
 	 * $script: ץ
 	 */
 	function image_update($script) {
-		$im = image_create();
 		$lock = image_lock();
+		$im = image_create();
 		interpret($im, $script);
 		$fname = get_blackboard_image_name();
 		image_save($im, $fname);
@@ -74,21 +83,28 @@
 	 */
 	function image_initialize() {
 		global $conf;
-		$im = imagecreate($conf['blackboard_width'], $conf['blackboard_height']);
-		imagecolortransparent($im, imagecolorallocate($im, 255, 255, 255));
+		//$im = imagecreate($conf['blackboard_width'], $conf['blackboard_height']);
+		//imagecolortransparent($im, imagecolorallocate($im, 255, 255, 255));
+		$im = image_create(true);
+		$lock = image_lock();
 		image_save($im, get_blackboard_image_name());
+		image_unlock($lock);
 	}
 
 	/*
 	 * ᡼֥Ȥ롣
 	 */
-	function image_create() {
+	function image_create($init=false) {
+		global $conf;
 		$fname = get_blackboard_image_name();
 		//if (!file_exists($fname) || exif_imagetype($fname) != IMAGETYPE_PNG) {
-		if (!file_exists($fname)) {
-			image_initialize();
+		if ($init or !file_exists($fname)) {
+			$im = imagecreate($conf['blackboard_width'], $conf['blackboard_height']);
+			imagecolortransparent($im, imagecolorallocate($im, 255, 255, 255));
+			return $im;
+		} else {
+			return imagecreatefrompng($fname);
 		}
-		return imagecreatefrompng($fname);
 	}
 
 	/*
@@ -100,7 +116,8 @@
 		global $conf;
 		$tmp = $conf['data_dir'] . "tmp.png";
 		imagepng($im, $tmp);
-		rename($tmp, $fname);
+		//rename($tmp, $fname);
+		copy($tmp, $fname);
 		// unlinkϤޤȤ褦ʤΤ̵
 		//unlink($tmp);
 	}
@@ -123,8 +140,6 @@
 	 * ߡΥեΥå
 	 */
 	function image_unlock($f) {
-		global $conf;
-		$dummy = $conf['data_dir'] . ".lock";
 		flock($f, LOCK_UN);
 		fclose($f);
 	}
@@ -133,19 +148,47 @@
 	 * 褹롣
 	 * $im: ᡼
 	 * $operands: 
-	 * $pen_color: ڥο
+	 * $pen_color: ڥο(r,g,b)
+	 * $pen_size: ڥΥ(ľ)
 	 */
-	function draw_line($im, $operands, $pen_color) {
+	function draw_line($im, $operands, $pen_color, $pen_size) {
+		$brush = create_brush($im, $pen_color, $pen_size);
 		$px = $operands[0];
 		$py = $operands[1];
 		$size = count($operands);
 		$i = ($size > 2) ?  2 : 0;
 		while ($i < $size - 1) {
-			imageline($im, $px, $py, $operands[$i], $operands[$i+1], $pen_color);
+			imageline($im, $px, $py, $operands[$i], $operands[$i+1], IMG_COLOR_BRUSHED);
 			$px = $operands[$i];
 			$py = $operands[$i+1];
 			$i += 2;
 		}
+		destroy_brush($brush);
+	}
+
+	/*
+	 * ꤵ줿Υ֥饷
+	 * $im: ᡼
+	 * $color: ֥饷ο(r,g,b)
+	 * $size: ֥饷Υ(ľ)
+	 */
+	function create_brush($im, $color, $size) {
+		$brush = imagecreate($size, $size);
+		imagecolortransparent($brush, imagecolorallocate($brush, 255, 255, 255));
+		list($r, $g, $b) = $color;		
+		$pen_color = imagecolorallocate($brush, $r, $g, $b);
+		imagearc($brush, $size/2,$size/2 ,$size,$size, 0,360, $pen_color);
+		imagefill($brush, $size/2, $size/2, $pen_color);
+		imagesetbrush($im, $brush);
+		return $brush;
+	}
+
+	/*
+	 * ֥饷Ѵ
+	 * $brush: Ѵ֥饷
+	 */
+	function destroy_brush($brush) {
+		imagedestroy($brush);
 	}
 
 	/*
@@ -156,13 +199,21 @@
 	function interpret($im, $script) {
 		$program = split(' ', $script);
 		$operands = array();
-		// ߤϥڥοϰ쿧
-		$pen_color = imagecolorallocate($im, 0xcc, 0xcc, 0xcc);
+		// ڥΥǥեȿ
+		$pen_color = array(0xcc, 0xcc, 0xcc);
+		// ڥΥǥեȥ
+		$pen_size = 1;
 		foreach ($program as $op) {
-			// ޤΤȤڥ졼ϥ饤lפ
+			// 饤l
 			if ($op == "l") {
-				draw_line($im, $operands, $pen_color);
+				draw_line($im, $operands, $pen_color, $pen_size);
 				$operands = array();
+			// Ѥc
+			} elseif ($op == 'c') {
+				$pen_color = array_splice($operands, -3);
+			// Ѥs
+			} elseif ($op == 's') {
+				$pen_size = array_pop($operands);
 			// ¾Ϥߤʿ
 			} else {
 				array_push($operands, $op);
@@ -221,11 +272,13 @@
 	 */
 	function plugin_initialize($name) {
 		global $conf;
+		plugin_load("plugins/" . $name . "/" . $name . ".inc");
+/*
 		if (in_array($name, $conf['plugins'])) {
-			plugin_load("plugins/" . $name . "/" . $name . ".inc");
 		} else {
 			error("̵ʥץ饰󤬻ꤵޤ");
 		}
+*/
 	}
 
 	/*
@@ -315,8 +368,35 @@
 	 */
 	function custom_conf_load($name) {
 		global $conf;
-		if (file_exists('conf/' . $name . '.ini')) {
-			$conf = array_merge($conf, parse_ini_file('conf/' . $name . '.ini'));
+		if (file_exists($conf['conf_dir'] . $name . '.ini')) {
+			$conf = array_merge($conf, parse_ini_file($conf['conf_dir'] . $name . '.ini'));
+		}
+	}
+
+	/*
+	 * ե¸
+	 * $name: ץ饰̾
+	 * $confs: (key:value)
+	 * $has_sections: Ĥɤ
+	 */
+	function custom_conf_save($name, $confs, $has_sections = false) {
+		global $conf;
+		if (!write_ini_file($confs, $conf['conf_dir'] . $name . '.ini', $has_sections)) {
+			error("ե񤭹߼");
+		}
+	}
+
+	/*
+	 * ե
+	 * $name: ץ饰̾
+	 */
+	function custom_conf_remove($name) {
+		global $conf;
+		$f = $conf['conf_dir'] . $name . '.ini';
+		if (file_exists($f)) {
+			if (!unlink($f)) {
+				error("ե");
+			}
 		}
 	}
 
@@ -330,4 +410,57 @@
 		exit;
 	}
 
+	/*
+	 * ץ饰
+	 */
+	class PluginInfo {
+		// ץ饰̾
+		var $name = "";
+		// 
+		var $author = "";
+		// 饤
+		var $license = "";
+		// С
+		var $version = "";
+		// ץ饰ۡڡ
+		var $homepage = "";
+		// ɲǽURL
+		var $download = "";
+		// ¸ץ饰
+		var $dependency = array();
+		// 
+		var $description = "";
+	}
+
+	/*
+	 * written by Jomel (k95vz5f02 AT sneakemail DOT com)
+	 * see http://jp.php.net/parse_ini_file
+	 */
+	function write_ini_file($assoc_arr, $path, $has_sections=FALSE) {
+		$content = "";
+
+		if ($has_sections) {
+			foreach ($assoc_arr as $key=>$elem) {
+				$content .= "[".$key."]\n";
+				foreach ($elem as $key2=>$elem2) {
+					$content .= $key2." = \"".$elem2."\"\n";
+				}
+			}
+		} else {
+			foreach ($assoc_arr as $key=>$elem) {
+				$content .= $key." = \"".$elem."\"\n";
+			}
+		}
+
+		if (!$handle = fopen($path, 'w')) {
+			return false;
+		}
+		if (!fwrite($handle, $content)) {
+			return false;
+		}
+		fclose($handle);
+		return true;
+	}
+
+
 ?>
